# Generated by Django 2.0 on 2017-12-22 16:49

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Campaign',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=256)),
                ('slug', models.SlugField()),
                ('short', models.TextField()),
                ('body', models.TextField()),
                ('goal', models.IntegerField()),
                ('end_date', models.DateField()),
            ],
        ),
        migrations.CreateModel(
            name='Pledge',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('email', models.EmailField(max_length=254)),
                ('name', models.CharField(max_length=256)),
                ('amount', models.IntegerField()),
                ('active', models.BooleanField(default=True)),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('updated', models.DateTimeField(auto_now=True)),
                ('campaign', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='raiseapp.Campaign')),
            ],
        ),
        migrations.CreateModel(
            name='Reminder',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('email', models.EmailField(max_length=254)),
                ('active', models.BooleanField(default=True)),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('updated', models.DateTimeField(auto_now=True)),
                ('campaign', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='raiseapp.Campaign')),
            ],
        ),
        migrations.CreateModel(
            name='Reward',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('amount', models.IntegerField()),
                ('description', models.TextField()),
                ('campaign', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='raiseapp.Campaign')),
            ],
        ),
    ]



# Generated by Django 2.0 on 2018-01-10 15:24

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('raiseapp', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='reward',
            name='timeframe',
            field=models.TextField(blank=True),
        ),
    ]



from django.urls import include, path

from . import views


urlpatterns = [
    path('<slug>/', include([
        path('', views.campaign, name='campaign'),
        path('pledge/', views.pledge, name='pledge'),
        path('remind/', views.reminder, name='remind'),
    ])),
]



from django.shortcuts import get_object_or_404, render, redirect
from django.contrib import messages

from . import models
from . import forms


def campaign(request, slug, template="campaign.html"):
    campaign = get_object_or_404(models.Campaign, slug=slug)

    return render(
        request,
        template,
        dict(
            campaign=campaign,
        ),
    )


def pledge(request, slug, post_pledge=None, template="pledge.html"):
    campaign = get_object_or_404(models.Campaign, slug=slug)

    if request.method == 'POST':
        pledge_form = forms.PledgeForm(request.POST)
        if pledge_form.is_valid():
            pledge = pledge_form.save(commit=False)
            pledge.campaign = campaign
            pledge.save()

            if post_pledge:
                post_pledge(request, pledge)

            messages.info(
                request,
                'Thanks for your pledge! You will receive an email shortly with more details.'
            )
            return redirect('campaign', campaign.slug)
    else:
        form = forms.PledgeForm()

    return render(
        request,
        template,
        dict(
            campaign=campaign,
            form=form,
        ),
    )


def reminder(request, slug, post_reminder=None, template="reminder.html"):
    campaign = get_object_or_404(models.Campaign, slug=slug)

    if request.method == 'POST':
        reminder_form = forms.ReminderForm(request.POST)
        if reminder_form.is_valid():
            reminder = reminder_form.save(commit=False)
            reminder.campaign = campaign
            reminder.save()
            if post_reminder:
                post_reminder(request, reminder)

            messages.info(
                request,
                'We will email you a reminder a few days before the campaign ends!'
            )
            return redirect('campaign', campaign.slug)

    else:
        form = forms.ReminderForm()

    return render(
        request,
        template,
        dict(
            campaign=campaign,
            form=form,
        ),
    )



# Generated by Django 2.0 on 2018-01-28 17:54

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('raiseapp', '0002_reward_timeframe'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='reward',
            options={'ordering': ['amount']},
        ),
        migrations.AddField(
            model_name='pledge',
            name='comments',
            field=models.TextField(blank=True),
        ),
    ]



from django.db import models
from django.utils.timezone import now


class Campaign(models.Model):
    title = models.CharField(max_length=256)
    slug = models.SlugField()
    short = models.TextField()
    body = models.TextField()

    goal = models.IntegerField()
    end_date = models.DateField()

    _raised = None

    @property
    def raised(self):
        if self._raised is None:
            pledge_sum = (
                self.pledge_set
                .exclude(active=False)
                .aggregate(models.Sum('amount'))
            )['amount__sum']

            if pledge_sum is None:
                self._raised = 0
            else:
                self._raised = pledge_sum

        return self._raised

    @property
    def percent(self):
        return self.raised / self.goal

    @property
    def display_percent(self):
        percent_value = self.percent * 100
        return '{:.0f}'.format(percent_value)

    @property
    def pledge_count(self):
        return self.pledge_set.exclude(active=False).count()

    @property
    def days_to_go(self):
        today = now().date()
        delta = self.end_date - today

        days = delta.days

        if days < 0:
            days = 0

        return days

    def __str__(self):
        return self.title


class Reward(models.Model):
    campaign = models.ForeignKey(Campaign, on_delete=models.CASCADE)
    amount = models.IntegerField()
    description = models.TextField()
    timeframe = models.TextField(blank=True)

    class Meta:
        ordering = ['amount']

    @property
    def display_amount(self):
        # TODO: If you wanted to do currencies you would do it here
        return str(self.amount)

    def __str__(self):
        return self.display_amount


class Reminder(models.Model):
    campaign = models.ForeignKey(Campaign, on_delete=models.CASCADE)
    email = models.EmailField()

    active = models.BooleanField(default=True)

    created = models.DateTimeField(auto_now_add=True)
    updated = models.DateTimeField(auto_now=True)

    def __str__(self):
        return self.email


class Pledge(models.Model):
    campaign = models.ForeignKey(Campaign, on_delete=models.CASCADE)
    email = models.EmailField()
    name = models.CharField(max_length=256)
    amount = models.IntegerField()

    comments = models.TextField(blank=True)

    active = models.BooleanField(default=True)

    created = models.DateTimeField(auto_now_add=True)
    updated = models.DateTimeField(auto_now=True)

    def __str__(self):
        return self.email



from setuptools import setup, find_packages
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='django-raise',
    version='0.0.6',
    description='Raise money from pledges',
    long_description=long_description,
    url='https://github.com/AvocadoCoop/django-raise',
    author='Avocado Co-op',
    author_email='albert@avocado.coop',
    classifiers=[
        'Development Status :: 3 - Alpha',
        # Pick your license as you wish
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
    ],
    packages=find_packages(exclude=['contrib', 'docs', 'tests']),
    package_data={},
)



from django import forms

from . import models


class ReminderForm(forms.ModelForm):
    class Meta:
        model = models.Reminder
        fields = [
            'email'
        ]


class PledgeForm(forms.ModelForm):
    class Meta:
        model = models.Pledge
        fields = [
            'email',
            'name',
            'amount',
        ]
